#!/usr/bin/env python3
"""
Generate opt file from the information entered in the DIA,
typically returning the vendor HV.

Usage:
    km3dia_generate_opt DOM_UPI FILENAME
    km3dia_generate_opt (-h | --help)
    km3dia_generate_opt --version

Options:
    DOM_UPI       DOM UPI in DIA DB (can be temporary one)
    FILENAME      Output OPT file
    -h --help     Show this screen.

"""
from docopt import docopt
import km3dia


def main():
    args = docopt(__doc__, version=km3dia.version)
    DOM_UPI = args["DOM_UPI"]
    output_file = args["FILENAME"]
    print(f"DOM UPI : {DOM_UPI}")
    print(f"\t OPT file will be saved here : {output_file}")
    km3dia.save_opt_file(DOM_UPI, output_file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Filename: core.py

"""
Database utilities.

"""
import requests
import json
import os

from km3db.logger import log

USER_INPUT = input
BASE_URL = "https://dia.km3net.de"
COOKIE_FILENAME = os.path.expanduser("~/.km3netdia_cookie")


class DBManager:
    """
    Low level class to access the DIA DB.

    The login information are provided by the user, and stored in a
    cookie for later usage. The user can extract this login info from
    the 8 last digits of the QR code that was provided to access the
    DIA website.

    Parameters:
    -----------
    url: str
       DIA server address. Default: "https://dia.km3net.de"
    container: str
       Return type from the wrapper. Can be `nt` for tupples, `pd` for pandas. Default: `nt`
    """

    def __init__(self, url=None, container="nt"):
        self._db_url = BASE_URL if url is None else ulr
        self._container = container
        self._session_cookie = None

    def get(self, url, OID=None):
        "Get HTML content"
        target_url = self._db_url + "/" + url
        print(target_url)
        x = None
        data = {"km3uname": OID}
        if data["km3uname"] is None:
            data["km3uname"] = self._request_session_cookie()

        try:
            x = requests.post(target_url, data, verify=False)

        except Exception as E:
            print(E)

        if x.text == "":
            log.warning(f'Empty answer from "{target_url}"')
            return None

        if self._container == "nt":
            return json.loads(x.text)

        elif self._container == "pd":
            import pandas

            return pandas.DataFrame(json.loads(x.text))

    @property
    def session_cookie(self):
        """Wrap the OID retrieving process"""
        if self._session_cookie is None:
            self._session_cookie = self._request_session_cookie()
        return self._session_cookie

    def _request_session_cookie(self):
        """Create OID cookie for permanent session."""
        if os.path.exists(COOKIE_FILENAME):
            log.info("Using cookie from %s", COOKIE_FILENAME)
            with open(COOKIE_FILENAME) as fobj:
                OID = fobj.read()
                return OID

        # The OID can also be set via the environment
        cookie = os.getenv("KM3NET_DIA_OID")
        if cookie is not None:
            log.info("Using OID from env ($KM3NET_DIA_OID)")
            return cookie

        # Last resort: we ask interactively
        OID = USER_INPUT("Please enter your KM3NeT DIA username: ")
        while self.check_OID_validity(OID) == False:
            OID = USER_INPUT("Please enter your KM3NeT DIA username: ")

        OID = "xxx." + OID
        log.info("Writing session cookie to %s", COOKIE_FILENAME)
        with open(COOKIE_FILENAME, "w") as fobj:
            fobj.write(f"xxx.{OID}")

        return OID

    def check_OID_validity(self, OID):
        """Check OID the validity to log to DIA"""
        result = self.get(
            f'/doquery.php?REQ=SELECT * FROM Operators WHERE KM3db_OID="{OID}";',
            OID="xxx." + OID,
        )
        if result is None:
            log.error("Hum, the OID appears to be wrong ...")
            return False
        return True

    def get_SQL(self, query):
        "Do SQL query to DIA DB"
        query_url = "/doquery.php?REQ=" + query
        if query_url[-1] != ";":
            query_url += ";"

        return self.get(query_url)



#!/usr/bin/env python3
# Filename: tools.py

import km3dia
import datetime
import re

"""
Collection of DOM and DU integration tools using DIA information
"""


def get_promisID(DOM_UPI):
    """Get promisID mapping for a specific DOM"""

    db = km3dia.DBManager(container="pd")
    df = db.get_SQL(
        f"SELECT * FROM DOM_INTEGRATION INNER JOIN Part_data ON DOM_INTEGRATION.PRODUPI = Part_data.PRODUPI  WHERE DOM_INTEGRATION.DOMUPI='{DOM_UPI}' AND LOCATE('3.4.2.3', Part_data.PRODUPI) > 0"
    )
    df = df.pivot(
        index=["DOMUPI", "POS", "PRODUPI"], columns="DataName", values="Datum"
    )
    df = df.reset_index()

    def POS2chanID(POS):
        chan = POS.split("(")[1]
        chan = chan.replace(")", "")
        chan = int(chan)
        return chan

    df["ChanID"] = df["POS"].apply(POS2chanID)
    df = df.set_index(["DOMUPI", "ChanID"]).sort_index()

    df["PMT_PROMISID"] = df["PMT_PROMISID"].str.replace("\t", "")
    return df


def save_opt_file(DOM_UPI, outfile, default_threshold=47):
    """Save to disk an optfile generated from DIA DB"""

    df = get_promisID(DOM_UPI)

    with open(outfile, "w") as fout:
        fout.write("# Optics configuration storage\n")
        fout.write("# Generated using km3dia\n")
        fout.write(f"# {datetime.datetime.now()}\n")
        fout.write(f"# {DOM_UPI}\n")

        for (DOM_UPI, ChanID), row in df.iterrows():
            fout.write(f"ch{ChanID}.id={row.PMT_PROMISID}\n")
            fout.write(f"ch{ChanID}.hv={row.PMT_HIGHVOLT}\n")
            fout.write(f"ch{ChanID}.ths={default_threshold}\n")



from pkg_resources import get_distribution, DistributionNotFound

version = get_distribution(__name__).version

from .core import DBManager
from .DOM_integration_summary import DOMIntegrationSummary
from .tools import get_promisID, save_opt_file



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))
import sys
import os
from datetime import date
import sphinx_rtd_theme
from pkg_resources import get_distribution

# -- Project information -----------------------------------------------------

version = get_distribution("km3dia").version
short_version = ".".join(version.split(".")[:2])
project = "km3dia {}".format(short_version)
copyright = "{0}, Valentin Pestel".format(date.today().year)
author = "Valentin Pestel"

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.viewcode",
    "autoapi.extension",
    "numpydoc",
    "sphinx_gallery.gen_gallery",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "version.py"]

# AutoAPI
autoapi_type = "python"
autoapi_dirs = ["../km3dia"]
autoapi_options = ["members", "undoc-members", "show-module-summary"]
autoapi_include_summaries = True

# Gallery
sphinx_gallery_conf = {
    "backreferences_dir": "gen_modules",
    "default_thumb_file": "_static/default_gallery_thumbnail.png",
    "examples_dirs": "../examples",  # path to your example scripts
    "gallery_dirs": "auto_examples",  # path to where to save gallery generated output
}

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]
html_title = "km3dia {}".format(version)



#!/usr/bin/env python
# Filename: setup.py
"""
The km3dia setup script.

"""
from setuptools import setup
import os


def read_requirements(kind):
    """Return a list of stripped lines from a file"""
    with open(os.path.join("requirements", kind + ".txt")) as fobj:
        return [l.strip() for l in fobj.readlines()]


try:
    with open("README.rst") as fh:
        long_description = fh.read()
except UnicodeDecodeError:
    long_description = "Python module providing access to DIA DB information."

setup(
    name="km3dia",
    url="https://git.km3net.de/km3py/km3dia",
    description="Python module providing access to DIA DB information.",
    long_description=long_description,
    author="Valentin Pestel",
    author_email="vpestel@km3net.de",
    packages=["km3dia"],
    include_package_data=True,
    platforms="any",
    setup_requires=["setuptools_scm"],
    use_scm_version=True,
    python_requires=">=3.6",
    install_requires=read_requirements("install"),
    extras_require={kind: read_requirements(kind) for kind in ["dev"]},
    entry_points={
        "console_scripts": ["km3dia_generate_opt=km3dia.cli.generate_opt:main"],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python",
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import unittest
import km3dia as kia


class test_reader_CSK(unittest.TestCase):
    def setUp(self):
        self.DB = kia.DBManager()

    def test_df(self):
        assert isinstance(self.DB, kia.DBManager)



#!/usr/bin/env python3
# Filename: DOM_integration_summary.py

"""
Module to get summary of DOM integration in a convenient format
"""

import km3dia
import pandas as pd
import numpy as np

from km3db.logger import log


def remove_dup_columns(frame):
    """Remove duplicated columns from a pandas DataFrame"""
    keep_names = set()
    keep_icols = list()
    for icol, name in enumerate(frame.columns):
        if name not in keep_names:
            keep_names.add(name)
        keep_icols.append(icol)
    return frame.iloc[:, keep_icols]


def measurement_name(tid, mid):
    """Format T_id and M_id into a string"""
    return "{:02d}_{:03d}".format(int(tid), int(mid))


class DOMIntegrationSummary:
    """
    Product a formated summary of the DOM integration
    information available in the DIA DB
    """

    def __init__(self, DIAManager=None):
        self.DIAManager = DIAManager
        if self.DIAManager is None:
            self.DIAManager = km3dia.DBManager(container="pd")

        self._test_results = None

    @property
    def test_results(self):
        """Wrapper for accessing test results"""
        if self._test_results is None:
            self._test_results = self._get_test_results()
        return self._test_results

    def _get_test_results(self):
        """Generate the test results DataFrame"""
        # Get all the measurements stored in the DIA DB
        req = "SELECT * FROM Test ts"
        req += " INNER JOIN Measure ms ON ts.E_T_id=ms.E_T_id"
        req += " INNER JOIN ASSIGNED_DOM ad on ad.UPI=ts.UPI"
        req += " INNER JOIN Bench bc on bc.B_id=ad.B_id"

        df = self.DIAManager.get_SQL(req)
        log.info(f"{df.shape[0]} measurement loaded.")

        # Link the measurement T_ID, M_ID and M_T_ID to measurement label
        measurement_map = {}
        df_tm = self.DIAManager.get_SQL("SELECT * FROM Test_Measures")
        df_tm = df_tm[pd.to_numeric(df_tm["T_id"], errors="coerce").notnull()]
        df_tm = df_tm.astype({"T_id": int, "M_id": int, "M_T_id": int})
        df_tm["Measurement"] = np.vectorize(measurement_name)(
            df_tm["T_id"], df_tm["M_id"]
        )

        for ind, row in df_tm.iterrows():
            if row["T_id"] == 3:
                df_tm.at[ind, "Name"] += " (func)"
            if row["T_id"] == 4:
                df_tm.at[ind, "Name"] += " (accept)"

        measure_name_dict = df_tm.set_index("Measurement")["Name"].to_dict()

        # Map the REPAIR states to initial steps
        T_id_mapReplace = {7: 2, 8: 3, 9: 4, 10: 5, 11: 6}

        # Pass from Narrow to wide format for measurement
        df = remove_dup_columns(df)
        df = df.dropna(subset=["VALUE"])
        df.replace({"T_id": T_id_mapReplace}, inplace=True)

        # Get rid of larger T_id, corresponding to BM integration
        df = df.astype({"T_id": int, "M_id": int})
        df = df[df["T_id"].astype(int) <= 6]

        df.sort_values("M_Time", inplace=True)
        df["Measurement"] = np.vectorize(measurement_name)(df["T_id"], df["M_id"])

        # Remove unexpected measurement.
        df = df[np.isin(df["Measurement"], df_tm["Measurement"])]

        self.measurements = df

        # Define the list of information we want to keep but not measurement-related
        DOM_info = [
            "PBS",
            "VARIANT",
            "VERSION",
            "SERIAL",
            "SiteID",
            "PhaseID",
            "Complete",
            "Registered",
            "NotConform",
            "Repair",
            "B_Details",
            "B_type",
        ]

        df_DOM = df.drop_duplicates(subset=["UPI"], keep="last").set_index("UPI")
        df_DOM = df_DOM[DOM_info]

        # Sorted in time, so keep the first/last result for each measurement
        df_first = df.drop_duplicates(subset=["UPI", "T_id"], keep="first")
        df_last = df.drop_duplicates(subset=["UPI", "T_id"], keep="last")
        df = df.drop_duplicates(subset=["UPI", "Measurement"], keep="last")

        step_to_time = {
            "functionnal": [3],
            "acceptance": [4, 5, 6],
        }

        # Use pivot to produce a wide version of the measurement table
        df = df.pivot(index="UPI", columns="Measurement", values="VALUE")

        for label, T_ids in step_to_time.items():
            first = df_first[np.isin(df_first["T_id"], T_ids)][
                ["UPI", "M_Time"]
            ].set_index("UPI")
            last = df_last[np.isin(df_first["T_id"], T_ids)][
                ["UPI", "M_Time"]
            ].set_index("UPI")
            df_DOM["time_first_" + label] = np.full(df_DOM.shape[0], "", dtype=object)
            df_DOM["time_last_" + label] = np.full(df_DOM.shape[0], "", dtype=object)
            df_DOM.loc[first.index, "time_first_" + label] = first["M_Time"]
            df_DOM.loc[last.index, "time_last_" + label] = last["M_Time"]
            df_DOM["time_first_" + label] = pd.to_datetime(
                df_DOM["time_first_" + label]
            )
            df_DOM["time_last_" + label] = pd.to_datetime(df_DOM["time_last_" + label])

        # Sort by label, to sort by step
        df.columns = np.sort(df.columns)
        df.rename(columns=measure_name_dict, inplace=True)

        # Merge generic info and measurement info
        df = pd.concat((df_DOM.loc[df.index], df), axis=1)
        df = df[df["SERIAL"].astype(int) > 2]
        df = df.reset_index().set_index(["SERIAL", "UPI"])

        return df
